<?php
/**
 * PHP - AJAX library for rapid development. 
 * 
 * 
 * Very simple library for rapid development, that provides: 
 *   - form creating
 *   - form validation
 *   - catching requests from AJAX frontend
 *   - json en/decoding
 *   - creating dom structure
 * 
 * @author	Martin Majlis (http://martin.m-core.net)
 * @version	0.1
 * @package Majax
 */

// ==============================================
// Global constants
// ==============================================

/**
 * Name of GET/POST key, where is stored request name. 
 */
define('MAJAX_REQUEST_NAME', 'majaxName');

// ==============================================
// Basic checks for form validation.
// ==============================================

/**
 * Minimal length of input value.
 */
define('MJX_C_MIN_LENGTH', 'minlength');

/**
 * Maximal length of input value.
 */
define('MJX_C_MAX_LENGTH', 'maxlength');

/**
 * Input value must be same as value of another element.
 */
define('MJX_C_SAME_AS', 'sameas');

/**
 * Input value must match regular expression (preg_match function is used).
 */
define('MJX_C_MATCH', 'match');

/**
 * Input value must not match regular expression (preg_match).
 */
define('MJX_C_NOT_MATCH', 'notmatch');

/**
 * Minimal value of input. 
 */
define('MJX_C_MIN_VALUE', 'minvalue');

/**
 * Maximal value of input.
 */
define('MJX_C_MAX_VALLUE', 'maxvalue');

/**
 * Input value must be from specified array.
 */
define('MJX_C_IN_VALUES', 'values'); 

/**
 * Input value must not be from specified array.
 */
define('MJX_C_NOT_IN_VALUES', 'notvalues');

/**
 * This rule is just for select element. Input value must be in known values.
 */
define('MJX_C_IN_SELECT', 'inselect');

// ==============================================
// List of predefined regulat expressions.
// ==============================================
 
/**
 * Regular expression for e-mail - http://mj.ucw.cz/download/net/checkaddr.pl
 */ 
define('MJX_R_MAIL', '~^([!#-\'*+.-9=?A-Z^-~-]+|"([ !#-\[\]-~]|\\[ -~])*")@([!#-\'*+-9=?A-Z^-~-]+\.[!#-\'*+.-9=?A-Z^-~-]+|\[([ -Z^-~]|\\[ -~])+(\.|\\\.)([ -Z^-~]|\\[ -~])+\])$~');

/**
 * Regular expression for phone number. 
 */
define('MJX_R_PHONE', '~(+(\d *){3} *)?(\d *){9}~');

/**
 * Regular expression for PSC.
 */
define('MJX_R_PSC', '~(\d *){5}~');

// ==============================================
// Input element types.
// ==============================================

/**
 * Input - type="text"
 */
define('MJX_T_TEXT', 'text');

/**
 * Input - type="password"
 */
define('MJX_T_PASSWORD', 'password');

/**
 * Input - type="radio"
 */
define('MJX_T_RADIO', 'radio');

/**
 * Input - type="checkbox"
 */
define('MJX_T_CHECKBOX', 'checkbox');

/**
 * Input - type="file"
 */
define('MJX_T_FILE', 'file');

/**
 * Input - type="submit"
 */
define('MJX_T_SUBMIT', 'submit');

/**
 * Input - type="reset"
 */
define('MJX_T_RESET', 'reset');

/**
 * Input - type="button"
 */
define('MJX_T_BUTTON', 'button');

/**
 * Input - type="image"
 */
define('MJX_T_IMAGE', 'image');

/**
 * Input - type="hidden"
 */
define('MJX_T_HIDDEN', 'hidden');

/**
 * Select
 */
define('MJX_T_SELECT', 'select');

/**
 * Textarea
 */
define('MJX_T_TEXTAREA', 'textarea');



/**
 * Core class for Majax library. 
 * 
 * @package Majax
 */
abstract class MajaxCore 
{
    
    /**
     * Access to undefined attribute is error.
     */
    function __get($name)
    {
        $class = get_class($this);
        user_error("Undefined property $class::\$$name", E_USER_ERROR);
    }

    /**
     * Access to undefined attribute is error.
     */
    function __set($name, $value)
    {
        $class = get_class($this);
        user_error("Undefined property $class::\$$name", E_USER_ERROR);
    }	
}

/**
 * Main class of Majax library, that provides functions for
 *  
 * @package Majax
 */
class Majax extends MajaxCore
{
	
	/**
	 * Instance of Majax - singleton.
	 * 
	 * @var	Majax
	 */
	private static $instance_ = null;
	
	/**
	 * Value, that is send. This value is assigned by <code>assign</code>
	 * and <code>assignXML</code> function. 
	 * 
	 * @var	string
	 */
	private $value_ = array();
	
	/**
	 * Registered functions, that are processed on Majax calls.
	 * 
	 * @var	array
	 */
	private $registered_;

	/**
	 * Instance of MajaxFormHandler. 
	 * 
	 * @var MajaxFormHandler
	 */
	private $form_;
	
	/**
	 * Creates instance of Majax and initializes. 
	 */
	private function Majax()
	{
		$this->form_ = MajaxFormHandler::Instance($this);
	}
	
	/**
	 * Returns Majax instance. 
	 * 
	 * @return Majax
	 */
	public static function & Instance()
	{
		if ( ! self::$instance_ ) { 
			self::$instance_ = new Majax();
			self::$instance_->initDefaults();
		}
		return self::$instance_;		
	}
	
	/**
	 * Initializes members.
	 */
	private function initDefaults()
	{
		$this->value_['__majax__']['submitOnSuccess'] = true;
		$this->value_['__majax__']['formValidity'] = true;
	}
	
	/**
	 * Assigns value, that will be sended. 
	 * 
	 * If parameter $encode is set to true, than value will be encoded by 
	 * jsonEncode function. Otherwise raw value is used. 
	 * 
	 * @param   $value	value
	 */
	public static function assign($value, $encode = true) { 
		$instance = self::Instance();
		if ( $encode ) { 
		    $instance->value_ = self::jsonEncode($value); 
		} else { 
		    $instance->value_ = $value;
		}
	}
	
	/**
	 * Assigns value, that will be sended. 
	 * 
	 * Value must be well-formed XML. Value is transformed by Majax::toDOM function.
	 * 
	 * @param   string  $xml    well-formed XML
	 * @see Majax::toDom()
	 */
	public static function assignXML($xml)
	{
		$instance = self::Instance();
		$instance->value_ = self::toDOM($xml);		
	}
	
	/**
	 * Sends (i. e. writes out) assigned values. 
	 * 
	 * Sends assigned values. If $exit is set to true, then after 
	 * sending exit. 
	 * If assigned value is not a string, then jsonEncode function is used. 
	 * 
	 * @param   bool    $exit   exit after sending
	 */
	public static function send($exit = true)
	{
		$instance = self::Instance();
		if (!is_string($instance->value_)) { 
			echo self::jsonEncode($instance->value_);
		} else { 
			echo $instance->value_;
		}
		if ($exit) { 
			exit;
		}
	}	
	
	/**
	 * Encodes string to JSON. 
	 * 
	 * @param   mixed   $value  value
	 * @return  string  encoded value
	 */
	public static function jsonEncode($value)
	{
		if (is_callable('json_encode')) { 
			return json_encode($value);
		} else { 
			return self::jsonEncode_($value);
		}
	} 
	
	/**
	 * Decodes JSON value to standart PHP type. 
	 *
	 * @param   string  $value  JSON value
	 * @return  mixed   php value
	 */
	public static function jsonDecode($value)
	{
		if (is_callable('json_decode')) { 
			return json_decode($value);
		} else { 
			return self::jsonDecode_($value);
		}		
	}
	
	/**
	 * Encodes PHP value to JSON value. 
	 *
	 * @param   mixed   $value  value
	 * @return  string  JSON value
	 */
	public function jsonEncode_($value) 
	{ 
	
		if ($value === NULL) { 
			return 'null';
		} elseif (is_bool($value)) { 
			return ($value ? 'true' : 'false');
		} elseif (is_string($value)) { 
			return '"'.addslashes($value).'"';			
		} elseif (is_numeric($value)) { 
			return (string) $value;
		} elseif (is_array($value)) { 
			return self::jsonEncodeArray_($value);
		} elseif (is_object($value)) { 
			return self::jsonEncodeObject_($value);
		}
		return '';
	} 

	/**
	 * Encodes PHP array or object to JSON. 
	 *
	 * Encodes PHP array ($isObject is false) or object ($isObject is true) 
	 * to JSON value. 
	 *
	 * @param   mixed   $value	PHP array or object
	 * @param   bool    $isObject	True, if value is object, false otherwise.
	 * @return  string  JSON value
	 */
	private function jsonEncodeArray_($value, $isObject = false)
	{
		$keys = array_keys($value);
		$len = count($value);
		$range = range(0, $len-1);
		if (($range == $keys || $len == 0) && !$isObject) { 
			// normal field, not associative
			$str = '[';
			for ($i = 0; $i < $len; $i++) {
				if ($i > 0) { 
					$str .= ',';
				}				
				$str .= self::jsonEncode_($value[$i]); 
			}
			$str .= ']';
			return $str;
		} else {
			// asociative array			
			$str = '{';
			for ($i = 0; $i < $len; $i++) { 
				if ($i > 0) { 
					$str .= ',';
				}				
			
				$str .= '"'.$keys[$i].'":'.
					self::jsonEncode_($value[$keys[$i]]);
			}
			$str .= '}';
			return $str;
		}
		
	}
	
	/**
	 * Encodes PHP object to JSON. 
	 *
	 * @param   object  $value  value
	 * @return  string  JSON
	 */
	private function jsonEncodeObject_($value) 
	{
		return self::jsonEncodeArray_(get_object_vars($value), true);
	}
	
	/**
	 * Decodes JSON value to PHP value. 
	 *
	 * @param   string  $json   JSON value
	 * @return  mixed   PHP value
	 */
	public function jsonDecode_($json)
	{
		$json = trim($json);
		
		if ($json == 'null') { 
			return NULL;
		} elseif ($json == 'true') { 
			return true;
		} elseif ($json == 'false') { 
			return false;
		} elseif ( is_numeric($json)) { 
			return $json;
		} elseif ( $json[0] == '{') { 
			return json_decode($json);
		} elseif ( $json[0] == '[') { 
			return json_decode($json);
		} else { 
			// string
			return substr($json, 1, -1);
		}
	}

	/**
	 * Converts well-formed XML to Javascript code, that creates this XML. 
	 * 
	 * If $xml is not well-formed exception is thrown. Return Javascript code, 
	 * that after evaluation creates DOM element representing $xml. This element 
	 * can be appended to existing structure. 
	 *
	 * @param   string  $xml    well-formed XML
	 * @return  string  Javascript code
	 */
	public static function toDOM($xml)
	{
		try { 
			
			$xml = str_replace("\n", "\\n", $xml);
			$xml = str_replace("\r", "\\r", $xml);
			$xml = str_replace("&nbsp;", " ", $xml);
						
			$d = new SimpleXMLElement($xml);
			
			$doc = new DOMDocument();
			$doc->loadXML($xml);
			
			$id = 0;
			$str = 'function MAJAX_DOM_FUNCTION() {';
			$stack = array();
			
			array_unshift($stack, array($doc->childNodes->item(0), -1));
			while (count($stack) > 0) {
				$act = array_shift($stack);
				$el = & $act[0];

				$type = $el->nodeType;
				if ($type == XML_ELEMENT_NODE) {	
					
					$str .= 'e'.$id.' = document.createElement("'.$el->nodeName.'");';

					// creates attributes
					$attributes = $el->attributes;
					
					foreach ($attributes as $at) { 
						$str .= 'e'.$id.'.setAttribute("'.$at->name.'", "'.self::addslashes($at->value).'");';
					}

					$tmpAr = array();
					$chCount = 0;
					foreach ($el->childNodes as $ch) { 
						$tmpAr[] = array($ch, $id);
						$chCount++;
					}
	
	
					while ($chCount > 0) { 
						array_unshift($stack, $tmpAr[--$chCount]);
					}
	
					if ($act[1] >= 0) { 
						$str .= 'e'.$act[1].'.appendChild(e'.$id.');';
					}
					$id++;
				} elseif ($type == XML_TEXT_NODE) {
					$str .= 't'.$id.' = document.createTextNode("'.self::addslashes($el->nodeValue).'");';
					$str .= 'e'.$act[1].'.appendChild(t'.$id.');';					
				} else { 
					echo $type;
					echo $act[1];
					exit;
				}
			}
			
			$str .= 'return e0;} MAJAX_DOM_FUNCTION();';
			return $str;
		} catch (exception $e) { 
			return false;
		}		
	}
	
	/**
	 * Registeres PHP function that handles AJAX request. 
	 * 
	 * Registres function, that is called when MAJAX request with name 
	 * $name is processed. If function is not specified, 
	 * function with name $function is called instead. 
	 *
	 * @param   string  $name   request name
	 * @param   mixed   $function	callback function
	 * @see MAJAX_REQUEST_NAME
	 */
	public static function register($name, $function = NULL)
	{
		if (empty($function)) { 
			$function = $name;
		}

		$instance = Majax::Instance();
		$instance->registered_[$name] = $function;	
	}
			
	
	/**
	 * Processes AJAX request. 
	 * 
	 * It's necessary to call this function to process MAJAX requests.
	 * 
	 * @see Majax::register
	 */
	public static function process()
	{
		$instance = Majax::Instance();
		
		$actName = '';
		if ($_GET[MAJAX_REQUEST_NAME]) { 
			$arg = $_GET;
			$actName = $_GET[MAJAX_REQUEST_NAME];
		} elseif ($_POST[MAJAX_REQUEST_NAME]) { 
			$arg = $_POST;
			$actName = $_POST[MAJAX_REQUEST_NAME];
		}

		$actName = $_GET[MAJAX_REQUEST_NAME] ? $_GET[MAJAX_REQUEST_NAME] : $_POST[MAJAX_REQUEST_NAME];
		
		$function = $instance->registered_[$actName];

		if ( !$function ) { 
			return false;
		}
		
		if ( is_array($function) ) { 
			if (is_object($function[0])) { 
				$function[0]->$function[1]($arg);
			} else { 
				$fce = create_function('$arg', 'return '.$function[0].'::'.$function[1].'($arg);');				
				$fce($arg);
			}		
		} else { 
			$function($arg);
		}
		return true;	
	}	
	
	/**
	 * Returns MajaxFormHandler instance. 
	 *
	 * @return MajaxFormHandler
	 */
	public static function Form()
	{
		$inst = Majax::Instance();
		return $inst->form_;
	}
	
	/**
	 * Adds slashes to string calling addcslashes function.
	 *
	 * @param   string  $str    string
	 * @return  string  string with slashes
	 */
	public static function addSlashes($str)
	{
		return addcslashes($str, '"\'');
	}
	
	
	public function & __get($name)
	{
		$stack = debug_backtrace();
		$l = $stack[1];
		if ($l['object'] && 
			is_a($l['object'], 'MajaxCore') && 
			!get_class($l['object']) != 'Majax') { 
				$nName = substr($name, 0, -1);
				return $this->$nName;
		} else { 
			parent::__get($name);			
		}
	}
}

/**
 * Provides basic interface for form handling. It's singleton pattern.
 * 
 * @package Majax
 */
class MajaxFormHandler extends MajaxCore
{
	private static $instance_ = NULL;
	private $majax_;
	private $majax = NULL;
	private $value_;

	/** 
	 * Creates instance of MajaxFormHandler.
	 * 
	 * @param   Majax   Majax instance
	 */
	private function MajaxFormHandler($majax)
	{
		$this->value_ = & $majax->value_P;
	}
	

	/**
	 * Returns MajaxFormHandler instance. 
	 * 
	 * @return  MajaxFormHandler
	 */
	public static function & Instance($majax = NULL)
	{
		if (!self::$instance_) { 
			if (get_class($majax) == 'Majax') {
				self::$instance_ = new MajaxFormHandler($majax);
			} 	
		}
		return self::$instance_;		 
	}	
	
	/**
	 * Registers handler for forms.
	 * 
	 * @param   string  $name   form name
	 * @param   mixed   $function	callback function
	 */
	public static function register($name, $function = NULL) { 
		$name = 'majaxForm_'	.$name;
		Majax::register($name, $function);
	}	

	/**
	 * Adds alert message <code>$msg</code>. When more than one message should be displayed, 
	 * they are displayd in one windows.
	 *
	 * @param   string  $msg    alert message
	 */
	public static function addAlert($msg)
	{
		$instance = Majax::Form();
		$instance->value_['__majax__']['alerts'][] = $msg;
	}	
	
	/**
	 * Sets, whether submited form is valid or not. 
	 * Default parameter is true. 
	 *
	 * @param   boolean $valid	form validity
	 */ 
	public static function setFormValidity($valid = true)
	{
		$instance = Majax::Form();
		$instance->value_['__majax__']['formValidity'] = $valid;	
	}
	
	/**
	 * Sets, whether submit form, if no error occured. 
	 * 
	 * @param   boolean $submit	whether submit
	 */
	public static function submitOnSuccess($submit = true)
	{		
		$instance = Majax::Form();
		$instance->value_['__majax__']['submitOnSuccess'] = (bool)$submit;			
	}
	
	/** 
	 * Creates new MajaxForm.
	 *
	 * @param   array   $arg    MajaxForm constructor parameter
	 * @see	MajaxForm
	 */
	public function create($arg)
	{
	    return new MajaxForm($arg);
	}
}

/**
 * General form element. Implements basic setters/getters. 
 *
 * @package Majax
 */
abstract class MajaxFormElement extends MajaxCore
{	
	/**
	 * Attributes
	 * @var mixed
	 */
    protected $attribute_ = array();
    protected $check_ = array();
    protected $valid_ = NULL;
	protected $parent_ = NULL; 
	protected $submittedValue_ = NULL;
	
	/**
	 * Creates element with parameters <code>$arg</code>. 
	 * 
	 * Parent element is not necessary for MajaxForm.
	 *
	 * @param	array	$arg	specificates properties
	 * @param	MajaxFormElement	$parent	parent element
	 */
	protected function MajaxFormElement($arg, $parent = NULL)
	{
		$this->parent_ = $parent;
		$this->initProperties($arg);		
	}

	/**
	 * Renders (i. e. displays) element. 
	 */
	abstract public function render();

	/*
	 * Returns true, if element is valid. 
	 * 
	 * If element wasn't validating, validation is called. 
	 *
	 * @return	true, if element is valid, false otherwise
	 */
	public function isValid()
	{
		if ($this->valid_ === NULL) { 
			$this->check();
		}
		return $this->valid_;
	}
	
	/**
	 * Init element property $name. 
	 *
	 * @param	string	$name	property name
	 * @param	mixed	$value	property value
	 *
	 * @see initChecks
	 */
	protected function doInitProperty(&$name, &$value)
	{
		if ($name == 'check') { 
			$this->initChecks($value);
			return true;
		}
		return false;
		
	}
		
	/**
	 * Init properties with $args. 
	 *
	 * @param	mixed	$args	init properties
	 */
	protected function initProperties(&$arg)
	{
		foreach ($arg as $k => $v) {
			if (!$this->doInitProperty($k, $v)) { 
				$this->attribute_[$k] = $v;
			}
		}
	}	
	
	/**
	 * Renders attributes ignoring attributes from $ignore parameter. 
	 *
	 * @param	array	$ignore	array of ignored attributes
	 */
	protected function renderAttributes($ignore = array())
	{
		$ignore[] = 'label';
		$ignore[] = 'check';
				
		$nameRendered = false;
		foreach ($this->attribute_ as $k => $v) { 
			if (!in_array($k, $ignore)) { 
				echo $k.'="'.$v.'" ';
				if ($k == 'name') { 
					$nameRendered = true;
				}
			}
		}
		if (!$nameRendered) { 
			echo 'name="'.$this->getName().'" ';
		}
				
	}
	
	/**
	 * Returns element's name. 
	 * 
	 * If name is not specified, id is returned. 
	 * 
	 * @return	element's name
	 */
	protected function getName()
	{
		if ($this->attribute_['name']) { 
			return $this->attribute_['name'];
		} else { 
			return $this->attribute_['id'];
		}
	}
	

	protected function getLabel()
	{
		if ($this->attribute_['label']) { 
			return $this->attribute_['label'];
		} else { 
			return $this->getName();
		}
	}
	
	public function setValue($val)
	{
		$this->attribute_['value'] = $val;
	}
	
	/// @TODO	opravit
	protected function initChecks($arg)
	{
		$this->check_ = $arg;
	}
	
	public function getAttribute($name)
	{
		return $this->attribute_[$name];
	}
	
	public function check($value)
	{
		$res = array(
			'valid' => true, 
			'alert' => array()
			);
			 
		if (is_array($this->check_)) { 
			foreach ($this->check_ as $k => $check) { 
				$al = '';
				if (is_array($check)) { 
					$rule = $check[0];
					$msg = $check[1];
				} else { 
					$rule = $check;
					$msg = $this->getName().' - '.$k.': '.$rule;
				}
				switch ($k) { 
					case MJX_C_MIN_LENGTH: 
						if (strlen($value) < $rule) { 
							$al = $msg;
						}	
						break;
					case MJX_C_MAX_LENGTH: 
						if (strlen($value) > $rule) { 
							$al = $msg;
						}
						break;
					case MJX_C_SAME_AS: 
						$f = $this->parent_->getElementByName($rule);
						//echo $this->getName().': '.$val.' vs '.$f->getName().': '.$f->getSubmittedValue().'<br />';
						if ($f && $value != $f->getSubmittedValue()) { 
							$al = $msg;
						}  
						break;
					case MJX_C_MATCH:
						if (!@preg_match($rule, $value)) { 
							$al = $msg;
						}
						break;
					case MJX_C_NOT_MATCH: 
						if (!@preg_match($rule, $value)) { 
							$al = $msg;
						}
						break;
					case MJX_C_MIN_VALUE: 
						if ($value < $rule) { 
							$al = $msg;
						}	
						break;
					case MJX_C_MAX_VALUE: 
						if ($value > $rule) { 
							$al = $msg;
						}
						break;	
					case MJX_C_IN_VALUES: 
						if (!in_array($value, $rule)) { 
							$al = $msg;
						}	
						break;
					case MJX_C_NOT_IN_VALUES: 
						if (in_array($value, $rule)) { 
							$al = $msg;
						}	
						break;		
					case MJX_C_IN_SELECT: 
						if ($this->attribute_['type'] == MJX_T_SELECT) { 
							$validValue = false;
							$vals = $this->attribute_['values'];
							reset($vals);
							foreach ($vals as $v) { 
								if ($v[0] == $value) { 
									$validValue = true;
									break;
								}
							}							
							reset($vals);
							if (!$validValue) {
								if (is_array($check) && !$rule) { 
									$al = $msg;
								} else { 
									$al = $rule;
								}
							} 
							break;							
						}
				
					default: 
						$al = $this->getName().': unknown check: '.$k.': '.var_export($v, true);		
				}
				if ($al) { 
					$res['alert'][] = $al;
					$res['valid'] = false;
				}
		
			}
		}
		
		return $res;
	}
	
	public function getSubmittedValue()
	{
		return $this->submittedValue_;
	}
	
	public function setSubmittedValud($val)
	{
		$this->submittedValue_ = $val;
	}
	
	
}

/**
 * @package Majax
 */
class MajaxForm extends MajaxFormElement 
{ 
	
	private $element_;
	private $processed_ = false;
	private $registered_ = false;
	
	public function MajaxForm($arg)
	{
		if (!is_array($arg[0]) || !is_array($arg[1])) { 
			throw new Exception('Invalid form constructor.');
		}
		$this->initProperties($arg[0]);
		$this->initElements($arg[1]);
		if (!$this->registered_) { 
			$this->register( 
				array(
					$this->attribute_['id'], 
					array($this, 'check')
				));
		}
	}
	
	private function initElements($element)
	{
		foreach ($element as $e) { 
			switch ($e['type']) { 
				case MJX_T_TEXT: 
					$el = new MajaxFormInputText($e, $this);
					break;
				case MJX_T_BUTTON: 
					$el = new MajaxFormInputButton($e, $this);
					break;
				case MJX_T_SUBMIT: 
					$el = new MajaxFormInputSubmit($e, $this);
					break;
				case MJX_T_FILE: 
					$el = new MajaxFormInputFile($e, $this);
					break;
				case MJX_T_RADIO:
					$el = new MajaxFormInputRadio($e, $this);
					break;
				case MJX_T_CHECKBOX: 
					$el = new MajaxFormInputCheckbox($e, $this);
					break;
				case MJX_T_PASSWORD:
					$el = new MajaxFormInputPassword($e, $this);
					break;
				case MJX_T_HIDDEN:
					$el = new MajaxFormInputHidden($e, $this);
					break;
				case MJX_T_RESET: 
					$el = new MajaxFormInputReset($e, $this);
					break;
				case MJX_T_IMAGE: 
					$el = new MajaxFormInputImage($e, $this);
					break;
				case MJX_T_SELECT: 
					$el = new MajaxFormSelect($e, $this);
					break;
				case MJX_T_TEXTAREA: 
					$el = new MajaxFormTextarea($e, $this);
					break;
				default: 
					$el = new MajaxFormUnknown($e, $this);
			}
			$this->element_[$el->getName()] = $el;
		}
	}
	
	public function render()
	{
		echo '<form ';
		$this->renderAttributes();
		echo '>';
		echo '<table>';
		foreach($this->element_ as &$el) { 
			echo '<tr>';
			if ($el->getAttribute('label') === '') { 
				echo '<td colspan="2">';
				$el->render();
			} else { 
				echo '<td>' .
						'<label for="'.$el->getName().'">'.$el->getLabel().'</label>' .
					'</td>' .
					'<td>';
						$el->render();
				echo    '</td>';
			}
			echo '</tr>';			
						
		}
		
		echo '</table>';
		echo '</form>';
		
	}
	
	private function register($arg)
	{
		if (is_array($arg)) { 
			Majax::Form()->register($arg[0], $arg[1]);
		} else { 
			Majax::Form()->register($arg);
		}
		$this->registered_ = true;
	}
	
	public function process()
	{
		if (!$this->processed_) { 
			Majax::process();
		}
	}
	
	protected function doInitProperty(&$name, &$value)
	{
		if ($name == 'register') {  
			$this->register($value);
			return true;
		} else { 
			return parent::doInitProperty($name, $value);
		} 
	}
	
	public function isProcessed()
	{
		return $this->processed_;
	}
	
	public function setProcessed()
	{
		$this->processed_ = true;
	}
	
	public function check($arg)
	{
		$this->valid_ = true;
		$this->setSubmittedValues($arg);
		reset($arg);
		foreach ($this->element_ as $el) { 
			$res = $el->check($arg[$el->getName()]);
			$this->valid_ &= $res['valid'];
			foreach ($res['alert'] as $al) { 
				Majax::Form()->addAlert($al);
			}
		}

		Majax::Form()->setFormValidity($this->valid_);
		//Majax::submitOnSuccess(true);
		
		Majax::send();			
	}
	
	public function isValid()
	{
		if ($this->valid_ === NULL) { 
			$this->process();
		}
		return $this->valid_;
	}
	
	public function setValues($values)
	{
		foreach ($this->element_ as $el) { 
			$t = $el->getAttribute('type');
			if (!in_array($t, array('submit', 'button'))) { 
				$el->setValue($values[$el->getName()]);
			}
		}
	}
	
	public function getElementByName($name)
	{
		return $this->element_[$name];
	}
	
	private function setSubmittedValues($arg)
	{
		foreach($arg as $k => $v) { 
			$f = $this->getElementByName($k);
			if ($f) { 
				$f->setSubmittedValud($v);
			}
		}
	}
} 

/**
 * @package Majax
 */
abstract class MajaxFormInput extends MajaxFormElement 
{ 
	public function render()
	{
		echo '<input ';
		$this->renderAttributes();
		echo '/>';
		
	}
	
	public function isValid()
	{
		return true;
	}	
	
	protected function doInitProperty(&$name, &$value)
	{
		return parent::doInitProperty($name, $value);
	}
}

/**
 * @package Majax
 */
class MajaxFormInputText extends MajaxFormInput
{
	
}

/**
 * @package Majax
 */
class MajaxFormInputPassword extends MajaxFormInput
{
	
}

/**
 * @package Majax
 */
class MajaxFormInputSubmit extends MajaxFormInput
{
	
}

/**
 * @package Majax
 */
class MajaxFormSelect extends MajaxFormElement
{
	public function render()
	{
		echo '<select ';
		$this->renderAttributes(array('value', 'values'));
		echo '>';
		
		$selected = false;
		$value = $this->attribute_['value'];
		
		foreach ($this->attribute_['values'] as $v) { 
			echo '<option value="'.$v[0].'" ';
			if ( ($v[2] && !isset($value) && !$selected) || 
				 ($v[1] == $value || $v[0] == $value) ) { 
				echo 'selected="selected" ';	 	
			} 
			echo '/>';
			echo $v[1] ? $v[1] : $v[0]; 
		}
		
		echo '</select>';	
		
	}
	
	protected function doInitProperty(&$name, &$value)
	{
		return parent::doInitProperty($name, $value);
	}	
}

/**
 * @package Majax
 */
class MajaxFormTextarea extends MajaxFormElement
{
	public function render()
	{
		echo '<textarea ';
		$this->renderAttributes(array('value'));
		echo '>'.$this->attribute_['value'].'</textarea>';		
	}
	
	protected function doInitProperty(&$name, &$value)
	{
		return parent::doInitProperty($name, $value);
	}	
}

/**
 * Unknown form element. 
 * 
 * Unknown form element - display just debug informations. 
 * @package Majax
 */
class MajaxFormUnknown extends MajaxFormElement
{
	
	protected function doInitProperty(&$name, &$value)
	{
		return parent::doInitProperty($name, $value);
	}	
	
	public function render()
	{		
		echo '<span style="border: 1px solid red; color: red;">';
		$this->renderAttributes();
		echo '</span>';	
	}
} 
?>
